//@author: dottore
//@help: standard constant shader
//@tags: color
//@credits: 

Texture2D inputTexture <string uiname="Texture";>;
SamplerState linearSampler <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tVP : LAYERVIEWPROJECTION;
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4x4 tColor <string uiname="Color Transform";>;
	float4 GlobCol : GLOBALCOLOR;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};

// =============================================================================
// VS ==========================================================================
// =============================================================================

struct VS_IN
{
    float4 PosO : POSITION;
};

struct VS_OUT
{
    float4 PosWVP : SV_Position;
};


VS_OUT VS(VS_IN In)
{
	VS_OUT Out;
	
	Out.PosWVP = mul(In.PosO, mul(tW,tVP));
	
	return Out;
}

// =============================================================================

struct VS_IN_TEX
{
    float4 PosO : POSITION;
	float4 uv: TEXCOORD0;
};

struct VS_OUT_TEX
{
    float4 PosWVP : SV_Position;
    float4 uv: TEXCOORD0;
};

VS_OUT_TEX VS_TEX(VS_IN_TEX In)
{
	VS_OUT_TEX Out;
	
	Out.PosWVP = mul(In.PosO,mul(tW,tVP));
	Out.uv = mul(In.uv, tTex);
	
	return Out;
}

// =============================================================================
// PS ==========================================================================
// =============================================================================

float4 PS(VS_OUT In): SV_Target
{
    float4 col = cAmb * GlobCol;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

// =============================================================================

float4 PS_TEX(VS_OUT_TEX In): SV_Target
{
    float4 col = inputTexture.Sample(linearSampler,In.uv.xy) * cAmb * GlobCol;
	col = mul(col, tColor);
	col.a *= Alpha;
    return col;
}

// =============================================================================
// TECHNIQUES ==================================================================
// =============================================================================


technique11 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}

technique11 ConstantTextures
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS_TEX() ) );
		SetPixelShader( CompileShader( ps_4_0, PS_TEX() ) );
	}
}




